import React, { useRef, useEffect } from 'react';

const DEFAULT_EVENT_PREFIX = "on";

const filterOutRichData = (props, overrideProps) => {
    return Object.keys(props).reduce((filteredProps, propKey) => {
        const value = props[propKey];
        if (!isRichData(value) || propKey === "children") {
            filteredProps[mapKeyToPropertyName(propKey, overrideProps)] = value;
        }
        return filteredProps;
    }, {});
};
const isRichData = (prop) => {
    return typeof prop === "object" || typeof prop === "function";
};
const mapKeyToEventName = (key, overrideProps) => {
    if (shouldKeyBeMapped(key, overrideProps)) {
        return overrideProps[key];
    }
    return key.substr(DEFAULT_EVENT_PREFIX.length).toLowerCase();
};
const mapKeyToPropertyName = (key, overrideProps) => {
    if (shouldKeyBeMapped(key, overrideProps)) {
        return overrideProps[key];
    }
    return key;
};
const shouldKeyBeMapped = (key, overrideProps) => {
    return overrideProps ? !!overrideProps[key] : false;
};

const adapt = (componentSelector, overrideProps) => {
    return (props) => {
        const webComponentRef = useRef(null);
        useEffect(() => {
            const eventListeners = {};
            const removeEventListeners = () => {
                for (let key in eventListeners) {
                    const handler = eventListeners[key];
                    webComponentRef.current.removeEventListener(mapKeyToEventName(key, overrideProps), handler);
                }
            };
            const setUpEventListeners = () => {
                for (let key in props) {
                    const handler = props[key];
                    if (key.indexOf(DEFAULT_EVENT_PREFIX) === -1 &&
                        !shouldKeyBeMapped(key, overrideProps)) {
                        continue;
                    }
                    if (typeof handler !== `function`) {
                        continue;
                    }
                    eventListeners[key] = handler;
                    webComponentRef.current.addEventListener(mapKeyToEventName(key, overrideProps), handler);
                }
            };
            const updatePropertiesForRichData = () => {
                for (let key in props) {
                    const data = props[key];
                    if (!isRichData(data) || key === "children") {
                        continue;
                    }
                    webComponentRef.current[mapKeyToPropertyName(key, overrideProps)] = data;
                }
            };
            setUpEventListeners();
            updatePropertiesForRichData();
            return () => {
                removeEventListeners();
            };
        });
        return React.createElement(componentSelector, Object.assign({ ref: (ref) => (webComponentRef.current = ref) }, filterOutRichData(props, overrideProps)));
    };
};

export { adapt };
